@@ -21,7 +21,7 @@
 #' @param maxt maximum \code{rt} allowed, used to stop integration problems (\code{prd} only).
 #' @param interval a vector containing the end-points of the interval to be searched for the desired quantiles (i.e., RTs) in \code{qdiffusion}. Default is \code{c(0, 10)}.
 #'
-#' @return \code{ddiffusion} gives the density, \code{pdiffusion} gives the distribution function, \code{qdiffusion} gives the quantile function (i.e., predicted RTs), and \code{rdiffusion} generates random response times and decisions (returning a \code{data.frame} with columns \code{rts} (numeric) and \code{response} (factor)).
+#' @return \code{ddiffusion} gives the density, \code{pdiffusion} gives the distribution function, \code{qdiffusion} gives the quantile function (i.e., predicted RTs), and \code{rdiffusion} generates random response times and decisions (returning a \code{data.frame} with columns \code{rt} (numeric) and \code{response} (factor)).
 #' 
 #' The length of the result is determined by \code{n} for \code{rrd}, and is equal to the length of \code{rt} for \code{drd} and \code{prd}.
 #' 
@@ -29,15 +29,23 @@
 #'
 #' @details The Ratcliff diffusion model (Ratcliff, 1978) is a mathematical model for two-choice discrimination tasks. It is based on the assumption that information is accumulated continuously until one of two decision thresholds is hit. For more information, see Voss, Rothermund, and Voss (2004), Voss, Nagler, and Lerche (2013), or Wagenmakers (2009).
 #' 
-#' All functions are fully vectorized acros all parameters as well as the boundary. This allows for trialwise parameters for each parameter. 
+#' All functions are fully vectorized across all parameters as well as the boundary. This allows for trialwise parameters for each parameter. 
+#' 
+#' \subsection{Quantile Function}{
+#' For the time being, the cumulative probability function (\code{pdiffusion}) uses numerical integration of \code{ddiffusion} via the R \code{\link{integrate}} function. While this is somewhat slow, it reliably produces correct results. Any problems in the numerical integration is passed on as a warning.
+#' }
 #' 
 #' \subsection{Quantile Function}{
 #' Due to the bivariate nature of the diffusion model, the diffusion processes reaching each boundary only return the defective CDF that does not reach 1. Only the sum of the CDF for both boundaries reaches 1. Therefore, \code{qdiffusion} can only return quantiles/RTs for any accumulator up to the maximal probability of that accumulator's CDF. This can be obtained by evaluating the CDF at a high value or \code{Inf} (the latter can be slow). See examples. 
 #' 
-#' Also note that quantiles (i.e., predicted RTs) are obtained by numerically minimizing the absolute difference between desired probabiliy and the value returned from \code{pdiffusion} using \code{\link{optimize}}. If the difference between the desired probability and probability corresponding to the returned quantile is above a certain threshold (currently 0.0001) no quantile is returned but \code{NA}. This can be either because the desired quantile is above the maximal probability for this accumulator or because the limits for the numerical integration are too small (default is \code{c(0, 10)}).
+#' Also note that quantiles (i.e., predicted RTs) are obtained by numerically minimizing the absolute difference between desired probability and the value returned from \code{pdiffusion} using \code{\link{optimize}}. If the difference between the desired probability and probability corresponding to the returned quantile is above a certain threshold (currently 0.0001) no quantile is returned but \code{NA}. This can be either because the desired quantile is above the maximal probability for this accumulator or because the limits for the numerical integration are too small (default is \code{c(0, 10)}).
 #' }
 #' 
-#' @note RTs need to be sorted (in increasing order) for \code{pdiffusion}.
+#' @note In the present code, the diffusion constant s is set to 1. This differs from, for example, the work of Roger Ratcliff who usually uses a diffusion constant of s=0.1. Estimates for a, z, and v depend on the chosen diffusion constant. These parameters can be transformed to the case of s = 1 by dividing the estimated values by the diffusion constant used for the estimation procedure. 
+#' 
+#' The parameterization of the non-decisional components, \code{t0} and \code{st0}, differs from the parameterization used by, for example, Andreas Voss or Roger Ratcliff in that \code{t0} is \emph{not} the lower bound of the uniform distribution of length \code{st0}, but its midpoint.
+#' 
+#' RTs need to be sorted (in increasing order) for \code{pdiffusion}.
 #' 
 #' @references Ratcliff, R. (1978). A theory of memory retrieval. \emph{Psychological Review}, 85(2), 59-108.
 #' 
@@ -54,7 +62,7 @@
 #'
 #' @name Diffusion
 #' @importFrom utils head
-#' @importFrom stats optimize
+#' @importFrom stats optimize uniroot
 #' @aliases diffusion
 #' 
 #' @example examples/examples.diffusion.R
@@ -161,6 +169,26 @@ ddiffusion <- function (rt, boundary = "upper",
   abs(densities)
 }
 
+.ddiffusion <- function (rt, boundary, params, precision)
+{
+  
+  nn <- length(rt)
+  # bind params to matrix
+  #params <- c(a, v, t0, d, sz, sv, st0, z, numeric_bounds)
+  
+  densities <- vector("numeric", nn)
+  densities <- .C("dfastdm_b", 
+                  as.integer (nn),                       # 1  IN:  number of densities
+                  as.vector  (params[1:8]),                     # 2  IN:  parameters
+                  as.vector  (rt),                         # 3  IN:  RTs
+                  as.double  (precision),                          # 4  IN:  precision
+                  as.integer (params[9]),                       # 5  IN:  boundary
+                  as.vector  (densities, mode="numeric")  # 6 OUT:  densities
+  )[[6]]
+  
+  abs(densities)
+}
+
 #' @rdname Diffusion
 #' @export
 pdiffusion <- function (rt, boundary = "upper", 
@@ -210,22 +238,84 @@ pdiffusion <- function (rt, boundary = "upper",
   uniques <- unique(params)
   for (i in seq_len(nrow(uniques))) {
     ok_rows <- apply(params, 1, identical, y = uniques[i,])
-    output <- .C("pfastdm_b", 
-                 as.integer (sum(ok_rows)),                          # 1  IN:  number of densities
-                 as.vector  (uniques[i,1:8]),                        # 2  IN:  parameters
-                 as.vector  (rt[ok_rows]),                            # 3  IN:  RTs
-                 as.double  (precision),                             # 4  IN:  precision
-                 as.integer (uniques[i,9]),                          # 5  IN:  boundary
-                 as.vector  (pvalues[ok_rows], mode="numeric")       # 6 OUT:  densities
-    )
-    pvalues[ok_rows] <- output[[6]]
+    for (j in seq_along(rt[ok_rows])) {
+      out <- integrate(f=.ddiffusion, 
+                       lower=if(j==1) 0 else rt[ok_rows][j-1],
+                       upper=rt[ok_rows][j],
+                       params=uniques[i,],
+                       precision=precision)
+      #browser()
+      if (out$message != "OK") warning(out$message, call. = FALSE)
+      pvalues[ok_rows][j] <- out$value
+    }
+    pvalues[ok_rows] <- cumsum(pvalues[ok_rows])
   }
   pvalues
 }
+# pdiffusion <- function (rt, boundary = "upper", 
+#                  a, v, t0, z = 0.5, d = 0, sz = 0, sv = 0, st0 = 0, 
+#                  precision = 3, maxt = 1e4) 
+# {
+#   if(any(missing(a), missing(v), missing(t0))) stop("a, v, and/or t0 must be supplied")
+# 
+#   rt[rt>maxt] <- maxt
+#   if(!all(rt == sort(rt)))  stop("rt needs to be sorted")
+# 
+#   # Convert boundaries to numeric
+#   nn <- length(rt)
+#   # Build parameter matrix  
+#   # Convert boundaries to numeric
+#   if (is.character(boundary)) {
+#     boundary <- match.arg(boundary, choices=c("upper", "lower"),several.ok = TRUE)
+#     numeric_bounds <- ifelse(boundary == "upper", 2L, 1L)
+#     }
+#   else {
+#     boundary <- as.numeric(boundary)
+#     if(any(!(boundary %in% 1:2))) stop("boundary needs to be either 'upper', 'lower', or as.numeric(boundary) %in% 1:2!")
+#     numeric_bounds <- as.integer(boundary)
+#   }
+#   numeric_bounds <- rep(numeric_bounds, length.out = nn)
+#   # all parameters brought to length of rt
+#   a <- rep(a, length.out = nn)
+#   v <- rep(v, length.out = nn)
+#   t0 <- rep(t0, length.out = nn)
+#   z <- rep(z, length.out = nn)
+#   d <- rep(d, length.out = nn)
+#   sz <- rep(sz, length.out = nn)
+#   sv <- rep(sv, length.out = nn)
+#   st0 <- rep(st0, length.out = nn)
+#   t0 <- recalc_t0 (t0, st0) 
+#   
+#   # bind params to matrix
+#   params <- cbind (a, v, t0, d, sz, sv, st0, z, numeric_bounds)
+#   
+#   
+#   # Check for illegal parameter values
+#   if(ncol(params)<9) stop("Not enough parameters supplied: probable attempt to pass NULL values?")
+#   if(!is.numeric(params)) stop("Parameters need to be numeric.")
+#   if (any(is.na(params)) || !all(is.finite(params))) stop("Parameters need to be numeric and finite.")
+#   
+#   pvalues <- vector("numeric", length=length(rt))    
+#   uniques <- unique(params)
+#   for (i in seq_len(nrow(uniques))) {
+#     ok_rows <- apply(params, 1, identical, y = uniques[i,])
+#     output <- .C("pfastdm_b", 
+#                  as.integer (sum(ok_rows)),                          # 1  IN:  number of densities
+#                  as.vector  (uniques[i,1:8]),                        # 2  IN:  parameters
+#                  as.vector  (rt[ok_rows]),                            # 3  IN:  RTs
+#                  as.double  (precision),                             # 4  IN:  precision
+#                  as.integer (uniques[i,9]),                          # 5  IN:  boundary
+#                  as.vector  (pvalues[ok_rows], mode="numeric")       # 6 OUT:  densities
+#     )
+#     pvalues[ok_rows] <- output[[6]]
+#   }
+#   pvalues
+# }
 
 
-inv_cdf_diffusion <- function(x, boundary, a, v, t0, z, d, sz, sv, st0, precision, maxt, value) {
-  abs(value - pdiffusion(rt=x, boundary=boundary, a=a, v=v, t0=t0, z=z, d=d, sz=sz, sv=sv, st0=st0, precision=precision, maxt=maxt))
+inv_cdf_diffusion <- function(x, boundary, a, v, t0, z, d, sz, sv, st0, precision, maxt, value, abs = TRUE) {
+  if (abs) abs(value - pdiffusion(rt=x, boundary=boundary, a=a, v=v, t0=t0, z=z, d=d, sz=sz, sv=sv, st0=st0, precision=precision, maxt=maxt))
+  else (value - pdiffusion(rt=x, boundary=boundary, a=a, v=v, t0=t0, z=z, d=d, sz=sz, sv=sv, st0=st0, precision=precision, maxt=maxt))
 }
 
 #' @rdname Diffusion
@@ -246,15 +336,26 @@ qdiffusion <- function (p, boundary = "upper",
   sz <- rep(unname(sz), length.out = nn)
   sv <- rep(unname(sv), length.out = nn)
   st0 <- rep(unname(st0), length.out = nn)
+  p <- unname(p)
   
   out <- vector("numeric", nn)
   for (i in seq_len(nn)) {
     tmp <- do.call(optimize, args = c(f=inv_cdf_diffusion, interval = list(interval), boundary=boundary[i], a=a[i], v=v[i], t0=t0[i], z=z[i], d=d[i], sz=sz[i], sv=sv[i], st0=st0[i], precision=precision, maxt=maxt, value =p[i], tol = .Machine$double.eps^0.5))
-    #browser()
     if (tmp$objective > 0.0001) {
-      warning("Cannot obtain RT that is less than 0.0001 away from desired p = ", p[i], ".\nIncrease interval or obtain for different boundary.", call. = FALSE)
-      out[i] <- NA
-    } else out[i] <- tmp[[1]]
+      tmp <- do.call(optimize, args = c(f=inv_cdf_diffusion, interval = list(c(min(interval),max(interval)/2)), boundary=boundary[i], a=a[i], v=v[i], t0=t0[i], z=z[i], d=d[i], sz=sz[i], sv=sv[i], st0=st0[i], precision=precision, maxt=maxt, value =p[i], tol = .Machine$double.eps^0.5))
+    }
+    if (tmp$objective > 0.0001) {
+      try({
+        uni_tmp <- do.call(uniroot, args = c(f=inv_cdf_diffusion, interval = list(interval), boundary=boundary[i], a=a[i], v=v[i], t0=t0[i], z=z[i], d=d[i], sz=sz[i], sv=sv[i], st0=st0[i], precision=precision, maxt=maxt, value =p[i], tol = .Machine$double.eps^0.5, abs = FALSE))
+      tmp$objective <- uni_tmp$f.root
+      tmp$minimum <- uni_tmp$root
+      }, silent = TRUE)
+    }
+    if (tmp$objective > 0.0001) {
+      tmp[["minimum"]] <- NA
+      warning("Cannot obtain RT that is less than 0.0001 away from desired p = ", p[i], ".\nIncrease/decrease interval or obtain for different boundary.", call. = FALSE)
+    }
+    out[i] <- tmp[["minimum"]]
   }
   return(out)
 }
