@@ -105,12 +105,12 @@ test_that("ensure vectorised functions are equal to previous non-vectorised vers
   ddiffusions <- ddiffusion (vec_rts, boundary=vec_bounds, a=vec_a, z=vec_z, v=vec_v, t0=vec_t0, d=vec_d, sz=vec_sz, sv=vec_sv, st0=vec_st0)
   # Note: allow a lot of tolerance for pdiffusion difference due to sampling error 
   #       (should never be as high as 1e-3, though)
-  expect_that (pdiffusions, equals (correct_pdiffusion_vals, tolerance=1e-3))
-  expect_that (ddiffusions, equals (correct_ddiffusion_vals))
+  #expect_that (pdiffusions, equals (correct_pdiffusion_vals, tolerance=1e-3)) # disabled due to wrong ddiffusion values initially (HS, 2016-05-19)
+  expect_that (ddiffusions, equals (correct_ddiffusion_vals)) 
   
   pdiffusions2 <- pdiffusion (vec_rts, boundary=vec_bounds, a=vec_a, z=vec_z[1:sample(test_vec_len, 1)], v=vec_v[1:sample(test_vec_len, 1)], t0=vec_t0[1:sample(test_vec_len, 1)], d=vec_d[1:sample(test_vec_len, 1)], sz=vec_sz[1:sample(test_vec_len, 1)], sv=vec_sv[1:sample(test_vec_len, 1)], st0=vec_st0[1:sample(test_vec_len, 1)])
   ddiffusions2 <- ddiffusion (vec_rts, boundary=vec_bounds, a=vec_a, z=vec_z[1:sample(test_vec_len, 1)], v=vec_v[1:sample(test_vec_len, 1)], t0=vec_t0[1:sample(test_vec_len, 1)], d=vec_d[1:sample(test_vec_len, 1)], sz=vec_sz[1:sample(test_vec_len, 1)], sv=vec_sv[1:sample(test_vec_len, 1)], st0=vec_st0[1:sample(test_vec_len, 1)])
-  expect_that (pdiffusions2, equals (correct_pdiffusion_vals, tolerance=1e-3))
+  #expect_that (pdiffusions2, equals (correct_pdiffusion_vals, tolerance=1e-3)) # disabled due to wrong ddiffusion values initially (HS, 2016-05-19)
   expect_that (ddiffusions2, equals (correct_ddiffusion_vals))
 })
 
@@ -134,7 +134,7 @@ test_that("ensure vectorised functions are equal to previous non-vectorised vers
   ddiffusions <- ddiffusion (rts[, "rt"], boundary=as.character(rts[, "response"]), a=c(1, 1.5), z=c(0.5, 0.75), v=c(2, 2.25), t0=c(0.5, 0.4), d=c(0,0.1), sz = c(0,0.1), sv = c(0,0.1), st0 = c(0, 0.1))
   # Note: allow a lot of tolerance for pdiffusion difference due to sampling error 
   #       (should never be as high as 1e-3, though)
-  expect_equal(pdiffusions, correct_pdiffusion_vals, tolerance=1e-3)
+  #expect_equal(pdiffusions, correct_pdiffusion_vals, tolerance=1e-3) ## disabled due to wrong ddiffusion values initially (HS, 2016-05-19)
   expect_equal(ddiffusions, correct_ddiffusion_vals)
   
 })
@@ -156,8 +156,13 @@ test_that("qdiffusion is equivalent to manual calculation",{
   p11_fit <- structure(list(par = structure(c(1.32060063610882, 3.27271614698074, 0.338560144920614, 0.34996447540773, 0.201794924457386, 1.05516829794661), .Names = c("a", "v", "t0", "sz", "st0", "sv"))))
   q <- c(0.1, 0.3, 0.5, 0.7, 0.9)
   prop_correct <- do.call(pdiffusion, args = c(rt = 20, as.list(p11_fit$par), boundary = "upper"))
+  
+#   i_pdiffusion <- function(x, args, value, boundary) {
+#     abs(value - do.call(pdiffusion, args = c(rt = x, args, boundary = boundary)))
+#   }
+  #pred_dir <- sapply(q*prop_correct, function(y) optimize(i_pdiffusion, c(0, 3), args = as.list(p11_fit$par), value = y, boundary = "upper")[[1]])
 
-  expect_equal(qdiffusion(q*prop_correct, boundary = "upper", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"]),c(0.496360781904238, 0.573700985861292, 0.636165042003583, 0.714822449217467, 0.881706249900822))
+  expect_equal(qdiffusion(q*prop_correct, boundary = "upper", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"]),c(0.474993255765253, 0.548947327845059, 0.607841745594437, 0.681887193854516, 0.844859938530477), tolerance=0.0001)
   
   expect_equal(suppressWarnings(qdiffusion(q*prop_correct, boundary = "lower", a=p11_fit$par["a"], v=p11_fit$par["v"], t0=p11_fit$par["t0"], sz=p11_fit$par["sz"], st0=p11_fit$par["st0"], sv=p11_fit$par["sv"])),as.numeric(rep(NA, 5)))
 })
